#!/usr/bin/env python3

import os
import re
import xml.etree.ElementTree as ET

# get function
def get(f="ros2_config.xml",pre='n'):

  cfg = {}

  mac = get_mac(last_three_octets=True,capitalize=False)
  cfg['self'] = pre+mac
  cfg['role'] = None

  if os.path.isfile(f):

    e = ET.parse(f).getroot()

    tmp = e.find('self')
    if tmp!=None:
      tmp = tmp.get('role')
      if tmp!=None:
        cfg['role'] = tmp

    cfg['slaves'] = {}
    for s in e.findall('slave'):

      tmp_name = s.get('class')
      tmp_mac  = s.get('id')

      cfg['slaves'][tmp_name] = tmp_mac
      # will not overwrite role for non-unique nodes

  return cfg


# get mac function
def get_mac(last_three_octets=True, keep_colons=False, capitalize=True):

  mac = "00:00:00:00:00:00"

  for root, dirs, files in os.walk("/sys/class/net/"):
    for name in dirs:
      # enp or wlp will work for linux pc for testing
      if re.match('(eth|enp|wlp)',name):
        fname = os.path.join(root,name,'address')
        #print(fname)
        with open(fname, 'r') as content_file:
          mac = content_file.read().strip()
        break

  if last_three_octets:
    mac = mac[-8:]

  if not keep_colons:
    mac = re.sub(':','',mac)

  if capitalize:
    mac = mac.upper()

  return mac




# MAIN
if __name__ == "__main__":

  # Test 1
  mac = get_mac()
  print(mac)

  # Test 2
  mac = get_mac(last_three_octets=False,keep_colons=True,capitalize=False)
  print(mac)
