#!/usr/bin/env python3

import xml.etree.ElementTree as ET
import xml.dom.minidom as minidom
import urllib.parse
import html

import rclpy
from rclpy.callback_groups import ReentrantCallbackGroup

from elphel_interfaces.srv import StrReqStrRes
import ros2_config

TIMEOUT = 5

def parse_command(string):

  msg = {}

  e = ET.fromstring(string)

  tmp = e.find('cmd')
  if tmp!=None and tmp.text!=None:
    # have to escape: &
    msg['cmd'] = html.escape(tmp.text)
  else:
    msg['cmd'] = 'state'

  tmp = e.find('targets')

  if tmp!=None and tmp.text!=None:
    msg['targets'] = tmp.text.split(",")
  else:
    msg['targets'] = clients

  return msg


# callback for broadcast commands
def cmd_callback(req,res):

  global g_node

  # put async function here?
  async def pass_cmd():
    global g_node
    nonlocal req
    nonlocal cli, did_run, did_get_result, cmd_result
    did_run = True
    try:
      requ = StrReqStrRes.Request()
      requ.request = req.request
      future = cli.call_async(requ)
      result = await future
      if result is not None:
        g_node.get_logger().info('Got response: '+result.response)
      else:
        g_node.get_logger().info('Service call failed %r' % (future.exception(),))
    finally:
      cmd_result = result
      did_get_result = True

  # default function
  cmd_func = pass_cmd

  #print("Raw command:")
  #print(req.request)

  q = parse_command(req.request)

  print("Parsed command:")
  print(q)

  if q['cmd']=='list':
    res.response = '<?xml version="1.0"?><document>'+'\n'.join(['<target>'+c+'</target>' for c in q['targets']])+'</document>'
    return res

  if q['cmd']=='state':
    cmd_func = pass_cmd
  
  cb_group = ReentrantCallbackGroup()
  reslist = []

  # serial operation. TODO: make parallel
  for t in q['targets']:
    # get target by name
    tgt = cfg['slaves'][t]
    cli = g_node.create_client(StrReqStrRes, tgt+"/cmd",callback_group=cb_group)

    did_run = False
    did_get_result = False
    cmdres = None
    cmd_result = None

    # if n/a
    if not cli.wait_for_service(timeout_sec=1.0):
      cmdres = "<response><node>"+t+"</node><state>dead</state></response>"
    else:
      # timer?!
      timer = g_node.create_timer(0.0, cmd_func, callback_group=cb_group)
      while rclpy.ok() and not did_run:
        rclpy.spin_once(g_node)
      # call timer callback only once
      if did_run:
        timer.cancel()
      # spin until it gets "blue"
      while rclpy.ok() and not did_get_result:
        rclpy.spin_once(g_node)
      cmdres  = "<response><node>"+t+"</node>"+cmd_result.response+"</response>"
    # store response
    reslist.append(cmdres)

  # convert to string
  res.response = '<?xml version="1.0"?><document>'+'\n'.join(reslist)+'</document>'
  return res


# global
g_node = None
clients = []
cfg = ros2_config.get(pre='master_')

nodename = cfg['self']

for k,v in cfg['slaves'].items():
  clients.append(k)

rclpy.init()
g_node = rclpy.create_node(nodename)

print('\nStarting master services:')
print('  '+nodename+'/cmd')
s1 = g_node.create_service(StrReqStrRes, nodename+'/cmd', cmd_callback)

while rclpy.ok():
  rclpy.spin_once(g_node)

# Destroy the service attached to the node explicitly
# (optional - otherwise it will be done automatically
# when the garbage collector destroys the node object)
g_node.destroy_service(srv)
rclpy.shutdown()




























